def someFunction(user, password="Admin"):
    print("Hi " + user)

def someFunction2(password):
    if password == "root":
        print("OK, logged in")

def noMatch(password):
    if password == '':
        print("No password!")

def NoMatch2(password):
    if password == "ajklawejrkl42348swfgkg":
        print("Nice password!")

doLogin(password="blerg")
password = "blerg"
d["password"] = "blerg"


def NoMatch3((a, b)):
    pass
